searchNodes=[{"doc":"EventStore allows you to define one or more event store modules to append, read, and subscribe to streams of events. It uses PostgreSQL (v9.5 or later) as the underlying storage engine. Defining an event store An event store module is defined in your own application as follows: defmodule MyApp.EventStore do use EventStore , otp_app : :my_app # Optional `init/1` function to modify config at runtime. def init ( config ) do { :ok , config } end end Where the configuration for the event store must be in your application environment, usually defined in config/config.exs : config :my_app , MyApp.EventStore , serializer : EventStore.JsonSerializer , username : &quot;postgres&quot; , password : &quot;postgres&quot; , database : &quot;eventstore&quot; , hostname : &quot;localhost&quot; Or use a URL to connect instead: config :my_app , MyApp.EventStore , serializer : EventStore.JsonSerializer , url : &quot;postgres://postgres:postgres@localhost/eventstore&quot; Note: To use an EventStore with Commanded you should configure the event store to use Commanded's JSON serializer which provides additional support for JSON decoding: config :my_app , MyApp.EventStore , serializer : Commanded.Serialization.JsonSerializer The event store module defines a start_link/1 function that needs to be invoked before using the event store. In general, this function is not called directly, but included as part of your application supervision tree. If your application was generated with a supervisor (by passing --sup to mix new ) you will have a lib/my_app/application.ex file containing the application start callback that defines and starts your supervisor. You just need to edit the start/2 function to start the event store in your application's supervisor: def start ( _type , _args ) do children = [ MyApp.EventStore ] opts = [ strategy : :one_for_one , name : MyApp.Supervisor ] Supervisor . start_link ( children , opts ) end Each event store module (e.g. MyApp.EventStore ) provides a public API to read events from and write events to an event stream, and subscribe to event notifications. Postgres schema By default the public schema will be used for event store tables. An event store can be configured to use an alternate Postgres schema: defmodule MyApp.EventStore do use EventStore , otp_app : :my_app , schema : &quot;schema_name&quot; end Or provide the schema as an option in the init/1 callback function: defmodule MyApp.EventStore do use EventStore , otp_app : :my_app def init ( config ) do { :ok , Keyword . put ( config , :schema , &quot;schema_name&quot; ) } end end Or define it in environment config when configuring the database connection settings: # config/config.exs config :my_app , MyApp.EventStore , schema : &quot;schema_name&quot; This feature allows you to define and start multiple event stores sharing a single Postgres database, but with their data isolated and segregated by schema. Note the mix event_store.&lt;task&gt; tasks to create, initialize, and drop an event store database will also handle creating and/or dropping the schema. Dynamic named event store An event store can be started multiple times by providing a name when starting. The name must be provided as an option to all event store operations to identify the correct instance. Example Define an event store: defmodule MyApp.EventStore do use EventStore , otp_app : :my_app end Start multiple instances of the event store, each with a unique name: { :ok , _pid } = EventStore . start_link ( name : :eventstore1 ) { :ok , _pid } = EventStore . start_link ( name : :eventstore2 ) { :ok , _pid } = EventStore . start_link ( name : :eventstore3 ) Use a dynamic event store by providing its name as an option to each function: :ok = EventStore . append_to_stream ( stream_uuid , expected_version , events , name : :eventstore1 ) { :ok , events } = EventStore . read_stream_forward ( stream_uuid , 0 , 1_000 , name : :eventstore1 ) Dynamic schemas This feature also allows you to start each event store instance using a different schema: { :ok , _pid } = EventStore . start_link ( name : :tenant1 , schema : &quot;tenant1&quot; ) { :ok , _pid } = EventStore . start_link ( name : :tenant2 , schema : &quot;tenant2&quot; ) Or start supervised: children = for tenant &lt;- [ :tenant1 , :tenant2 , :tenant3 ] do { MyApp.EventStore , name : tenant , schema : &quot; \#{ tenant } &quot; } end opts = [ strategy : :one_for_one , name : MyApp.Supervisor ] Supervisor . start_link ( children , opts ) The above can be used for multi-tenancy where the data for each tenant is stored in a separate, isolated schema. Shared database connection pools By default each event store will start its own Postgrex database connection pool. The size of the pool is configured with the pool_size config option. When you have multiple event stores running you will also end up with multiple connection pools. If they are all connecting to the same physical Postgres database then it can be useful to share a single pool amongst all event stores. Use the shared_connection_pool config option to specify a name for the shared connection pool. Then configure the event stores you'd like to share the pool with the same name. This can be done in config: # config/config.exs config :my_app , MyApp.EventStore , shared_connection_pool : :shared_pool Or when starting the event stores, such as via a Supervisor : Supervisor . start_link ( [ { MyApp.EventStore , name : :eventstore1 , shared_connection_pool : :shared_pool } , { MyApp.EventStore , name : :eventstore2 , shared_connection_pool : :shared_pool } , { MyApp.EventStore , name : :eventstore3 , shared_connection_pool : :shared_pool } ] , opts ) Using an existing database connection or transaction In some situations you might want to execute the event store operations using an existing Postgres database connection or transaction. For instance, if you want to persist changes to one or more other tables, such as a read-model projection. To do this you can provide a Postgrex connection process or transaction as a :conn option to any of the supported EventStore functions. { :ok , pid } = Postgrex . start_link ( config ) Postgrex . transaction ( pid , fn conn -&gt; :ok = EventStore . append_to_stream ( stream_uuid , expected_version , events , conn : conn ) end ) This can also be used with an Ecto Repo which is configured to use the Postgres SQL adapter. The connection process may be looked up as follows: Repo . transaction ( fn -&gt; %{ pid : pool } = Ecto.Adapter . lookup_meta ( Repo ) conn = Process . get ( { Ecto.Adapters.SQL , pool } ) :ok = EventStore . append_to_stream ( stream_uuid , expected_version , events , conn : conn ) end ) Guides Please refer to the following guides to learn more: Getting started Usage Subscriptions Running on a cluster of nodes Event serialization Upgrading an existing EventStore database","ref":"EventStore.html","title":"EventStore","type":"behaviour"},{"doc":"Acknowledge receipt of the given events received from a subscription. Accepts a single EventStore.RecordedEvent struct, a list of EventStore.RecordedEvent s, or the event number of the recorded event to acknowledge.","ref":"EventStore.html#c:ack/2","title":"EventStore.ack/2","type":"callback"},{"doc":"Returns all running EventStore instances. Note that order is not guaranteed.","ref":"EventStore.html#all_instances/0","title":"EventStore.all_instances/0","type":"function"},{"doc":"Append one or more events to a stream atomically. stream_uuid is used to uniquely identify a stream. expected_version is used for optimistic concurrency checks. You can provide a non-negative integer to specify the expected stream version. This is used to ensure you can only append to the stream if it is at exactly that version. You can also provide one of the following values to alter the concurrency check behaviour: :any_version - No concurrency checking and allow any stream version (including no stream). :no_stream - Ensure the stream does not exist. :stream_exists - Ensure the stream exists. events is a list of %EventStore.EventData{} structs. opts an optional keyword list containing: name the name of the event store if provided to start_link/1 . timeout an optional timeout for the database transaction, in milliseconds. Defaults to 15,000ms. Returns :ok on success, or an {:error, reason} tagged tuple. The returned error may be due to one of the following reasons: {:error, :wrong_expected_version} when the actual stream version differs from the provided expected version. {:error, :stream_exists} when the stream exists, but expected version was :no_stream . {:error, :stream_not_found} when the stream does not exist, but expected version was :stream_exists .","ref":"EventStore.html#c:append_to_stream/4","title":"EventStore.append_to_stream/4","type":"callback"},{"doc":"Returns the event store configuration stored in the :otp_app environment.","ref":"EventStore.html#c:config/0","title":"EventStore.config/0","type":"callback"},{"doc":"Delete an existing persistent subscription to all streams. subscription_name is used to identify the existing subscription to remove. Returns :ok on success.","ref":"EventStore.html#c:delete_all_streams_subscription/2","title":"EventStore.delete_all_streams_subscription/2","type":"callback"},{"doc":"Delete a previously recorded snapshop for a given source. Returns :ok on success, or when the snapshot does not exist.","ref":"EventStore.html#c:delete_snapshot/2","title":"EventStore.delete_snapshot/2","type":"callback"},{"doc":"Delete an existing stream. stream_uuid identity of the stream to be deleted. expected_version is used for optimistic concurrency checking. You can provide a non-negative integer to specify the expected stream version. This is used to ensure you can only delete a stream if it is at exactly that version. You can also provide one of the following values to alter the concurrency checking behaviour: :any_version - No concurrency check, allow any stream version. :stream_exists - Ensure the stream exists, at any version. type - used to indicate how the stream is deleted: :soft - the stream is marked as deleted, but no events are removed. :hard - the stream and its events are permanently deleted from the database. Soft deletion is the default if the type is not provided. Returns :ok on success or an error tagged tuple on failure. Soft delete Will mark the stream as deleted, but will not delete its events. Events from soft deleted streams will still appear in the globally ordered all events ( $all ) stream and in any linked streams. A soft deleted stream cannot be read nor appended to. Subscriptions to the deleted stream will not receive any events but subscriptions containing linked events from the deleted stream, such as the global all events stream, will still receive events from the deleted stream. Hard delete Will permanently delete the stream and its events. This is irreversible and will remove data . Events will be removed from the globally ordered all events stream and any linked streams. After being hard deleted, a stream can later be appended to and read as if it had never existed. Examples Soft delete a stream Delete a stream at any version: :ok = MyApp.EventStore . delete_stream ( &quot;stream1&quot; , :any_version , :soft ) Delete a stream at an expected version: :ok = MyApp.EventStore . delete_stream ( &quot;stream2&quot; , 3 , :soft ) Delete stream will use soft delete by default so you can omit the type: :ok = MyApp.EventStore . delete_stream ( &quot;stream1&quot; , :any_version ) Hard delete a stream Since hard deletes are destructive and irreversible they are disabled by default. To use hard deletes you must first enable them for the event store: defmodule MyApp.EventStore do use EventStore , otp_app : :my_app , enable_hard_deletes : true end Or via config: # config/config.exs config :my_app , MyApp.EventStore , enable_hard_deletes : true Hard delete a stream at any version: :ok = MyApp.EventStore . delete_stream ( &quot;stream1&quot; , :any_version , :hard ) Hard delete a stream that should exist: :ok = MyApp.EventStore . delete_stream ( &quot;stream2&quot; , :stream_exists , :hard )","ref":"EventStore.html#c:delete_stream/4","title":"EventStore.delete_stream/4","type":"callback"},{"doc":"Delete an existing persistent subscription. stream_uuid is the stream the subscription is subscribed to. subscription_name is used to identify the existing subscription to remove. Returns :ok on success.","ref":"EventStore.html#c:delete_subscription/3","title":"EventStore.delete_subscription/3","type":"callback"},{"doc":"A callback executed when the event store starts or when configuration is read. It must return {:ok, keyword} with the updated list of configuration.","ref":"EventStore.html#c:init/1","title":"EventStore.init/1","type":"callback"},{"doc":"Link one or more existing events to another stream. Allows you to construct streams containing events already appended to any other stream. This is more efficient than copying events between streams since only a reference to the existing event is created. stream_uuid is used to uniquely identify the target stream. expected_version is used for optimistic concurrency checks. You can provide a non-negative integer to specify the expected stream version. This is used to ensure you can only append to the stream if it is at exactly that version. You can also provide one of the following values to affect the concurrency check behaviour: :any_version - No concurrency checking; allow any stream version (including no stream). :no_stream - Ensure the stream does not exist. :stream_exists - Ensure the stream exists. events_or_event_ids is a list of %EventStore.EventData{} structs or event ids. opts an optional keyword list containing: name the name of the event store if provided to start_link/1 . timeout an optional timeout for the database transaction, in milliseconds. Defaults to 15,000ms. Returns :ok on success, or an {:error, reason} tagged tuple. The returned error may be due to one of the following reasons: {:error, :wrong_expected_version} when the actual stream version differs from the provided expected version. {:error, :stream_exists} when the stream exists, but expected version was :no_stream . {:error, :stream_not_found} when the stream does not exist, but expected version was :stream_exists .","ref":"EventStore.html#c:link_to_stream/4","title":"EventStore.link_to_stream/4","type":"callback"},{"doc":"Paginate all streams. opts an optional keyword list containing: page_size the total number of streams per page. Defaults to 50. page_number the current page number. Defaults to page 1. search search for a stream by its identity. sort_by sort the streams by the given field. Defaults to sorting by the stream's internal id ( :stream_id field) sort_dir direction to sort streams by, either :asc or :desc . Defaults to :asc . name the name of the event store if provided to start_link/1 . Defaults to the event store module name (e.g. MyApp.EventStore ). timeout an optional timeout for the database transaction, in milliseconds. Defaults to 15,000ms. Returns an {:ok, page} result containing a list of StreamInfo structs, or an error tagged tuple on failure. Example alias EventStore.Page { :ok , % Page { entries : streams } } = MyApp.EventStore . paginate_streams ( )","ref":"EventStore.html#c:paginate_streams/1","title":"EventStore.paginate_streams/1","type":"callback"},{"doc":"Reads the requested number of events from all streams in the reverse order from which they were originally written. start_version optionally, the stream version of the first event to read. Use -1 to indicate starting from the end of the stream. Defaults to the end of the stream if not set. count optionally, the maximum number of events to read. Defaults to returning 1,000 events from all streams. opts an optional keyword list containing: name the name of the event store if provided to start_link/1 . timeout an optional timeout for the database transaction, in milliseconds. Defaults to 15,000ms.","ref":"EventStore.html#c:read_all_streams_backward/3","title":"EventStore.read_all_streams_backward/3","type":"callback"},{"doc":"Reads the requested number of events from all streams in the order in which they were originally written. start_version optionally, the stream version of the first event to read. Defaults to the beginning of the stream if not set. count optionally, the maximum number of events to read. Defaults to returning 1,000 events from all streams. opts an optional keyword list containing: name the name of the event store if provided to start_link/1 . timeout an optional timeout for the database transaction, in milliseconds. Defaults to 15,000ms.","ref":"EventStore.html#c:read_all_streams_forward/3","title":"EventStore.read_all_streams_forward/3","type":"callback"},{"doc":"Read a snapshot, if available, for a given source. Returns {:ok, %EventStore.Snapshots.SnapshotData{}} on success, or {:error, :snapshot_not_found} when unavailable.","ref":"EventStore.html#c:read_snapshot/2","title":"EventStore.read_snapshot/2","type":"callback"},{"doc":"Reads the requested number of events from the given stream in the reverse order from which they were originally written. stream_uuid is used to uniquely identify a stream. start_version optionally, the stream version of the first event to read. Use -1 to indicate starting from the end of the stream. Defaults to the end of the stream if not set. count optionally, the maximum number of events to read. Defaults to to returning 1,000 events from the stream. opts an optional keyword list containing: name the name of the event store if provided to start_link/1 . timeout an optional timeout for the database transaction, in milliseconds. Defaults to 15,000ms.","ref":"EventStore.html#c:read_stream_backward/4","title":"EventStore.read_stream_backward/4","type":"callback"},{"doc":"Reads the requested number of events from the given stream in the order in which they were originally written. stream_uuid is used to uniquely identify a stream. start_version optionally, the stream version of the first event to read. Defaults to the beginning of the stream if not set. count optionally, the maximum number of events to read. Defaults to to returning 1,000 events from the stream. opts an optional keyword list containing: name the name of the event store if provided to start_link/1 . timeout an optional timeout for the database transaction, in milliseconds. Defaults to 15,000ms.","ref":"EventStore.html#c:read_stream_forward/4","title":"EventStore.read_stream_forward/4","type":"callback"},{"doc":"Record a snapshot of the data and metadata for a given source. Returns :ok on success.","ref":"EventStore.html#c:record_snapshot/2","title":"EventStore.record_snapshot/2","type":"callback"},{"doc":"Starts any connection pooling or supervision and return {:ok, pid} or just :ok if nothing needs to be done. Returns {:error, {:already_started, pid}} if the event store is already started or {:error, term} in case anything else goes wrong.","ref":"EventStore.html#c:start_link/1","title":"EventStore.start_link/1","type":"callback"},{"doc":"Shuts down the event store.","ref":"EventStore.html#c:stop/2","title":"EventStore.stop/2","type":"callback"},{"doc":"Streams events from all streams in the reverse order from which they were originally written. start_version optionally, the stream version of the first event to read. Use -1 to indicate starting from the end of the stream. Defaults to the end of the stream if not set. opts an optional keyword list containing: name the name of the event store if provided to start_link/1 . timeout an optional timeout for the database transaction, in milliseconds. Defaults to 15,000ms. read_batch_size optionally, the number of events to read at a time from storage. Defaults to reading 1,000 events per batch.","ref":"EventStore.html#c:stream_all_backward/2","title":"EventStore.stream_all_backward/2","type":"callback"},{"doc":"Streams events from all streams in the order in which they were originally written. start_version optionally, the stream version of the first event to read. Defaults to the beginning of the stream if not set. opts an optional keyword list containing: name the name of the event store if provided to start_link/1 . timeout an optional timeout for the database transaction, in milliseconds. Defaults to 15,000ms. read_batch_size optionally, the number of events to read at a time from storage. Defaults to reading 1,000 events per batch.","ref":"EventStore.html#c:stream_all_forward/2","title":"EventStore.stream_all_forward/2","type":"callback"},{"doc":"Streams events from the given stream in the reverse order from which they were originally written. start_version optionally, the stream version of the first event to read. Use -1 to indicate starting from the end of the stream. Defaults to the end of the stream if not set. opts an optional keyword list containing: name the name of the event store if provided to start_link/1 . timeout an optional timeout for the database transaction, in milliseconds. Defaults to 15,000ms. read_batch_size optionally, the number of events to read at a time from storage. Defaults to reading 1,000 events per batch.","ref":"EventStore.html#c:stream_backward/3","title":"EventStore.stream_backward/3","type":"callback"},{"doc":"Streams events from the given stream in the order in which they were originally written. start_version optionally, the stream version of the first event to read. Defaults to the beginning of the stream if not set. opts an optional keyword list containing: name the name of the event store if provided to start_link/1 . timeout an optional timeout for the database transaction, in milliseconds. Defaults to 15,000ms. read_batch_size optionally, the number of events to read at a time from storage. Defaults to reading 1,000 events per batch.","ref":"EventStore.html#c:stream_forward/3","title":"EventStore.stream_forward/3","type":"callback"},{"doc":"Get basic information about a stream, including its version, status, and created date. opts an optional keyword list containing: name the name of the event store if provided to start_link/1 . timeout an optional timeout for the database transaction, in milliseconds. Defaults to 15,000ms. Returns {:ok, StreamInfo.t()} on success, or an {:error, reason} tagged tuple. The returned error may be due to one of the following reasons: - ` { :error , :stream_not_found } ` when the stream does not exist . - ` { :error , :stream_deleted } ` when the stream was soft deleted . Example alias EventStore.Streams.StreamInfo { :ok , % StreamInfo { stream_version : stream_version } } = MyApp.EventStore . stream_info ( &quot;stream-1234&quot; )","ref":"EventStore.html#c:stream_info/2","title":"EventStore.stream_info/2","type":"callback"},{"doc":"Create a transient subscription to a given stream. stream_uuid is the stream to subscribe to. Use the $all identifier to subscribe to events from all streams. opts is an optional map providing additional subscription configuration: name the name of the event store if provided to start_link/1 . selector to define a function to filter each event, i.e. returns only those elements for which fun returns a truthy value mapper to define a function to map each recorded event before sending to the subscriber. The calling process will be notified whenever new events are appended to the given stream_uuid . As the subscription is transient you do not need to acknowledge receipt of each event. The subscriber process will miss any events if it is restarted and resubscribes. If you need a persistent subscription with guaranteed at-least-once event delivery and back-pressure you should use EventStore.subscribe_to_stream/4 . Notification message Events will be sent to the subscriber, in batches, as {:events, events} where events is a collection of EventStore.RecordedEvent structs. Example { :ok , subscription } = EventStore . subscribe ( stream_uuid ) # receive first batch of events receive do { :events , events } -&gt; IO . puts &quot;Received events: &quot; &lt;&gt; inspect ( events ) end","ref":"EventStore.html#c:subscribe/2","title":"EventStore.subscribe/2","type":"callback"},{"doc":"Create a subscription to all streams. By default the subscription is persistent. The subscriber process will be notified of each batch of events appended to any stream. subscription_name is used to uniquely identify the subscription. subscriber is a process that will be sent {:events, events} notification messages. opts is an optional map providing additional subscription configuration: name the name of the event store if provided to start_link/1 . start_from is a pointer to the first event to receive. It must be one of: :origin for all events from the start of the stream (default). :current for any new events appended to the stream after the subscription has been created. any positive integer for an event id to receive events after that exact event. selector to define a function to filter each event, i.e. returns only those elements for which fun returns a truthy value mapper to define a function to map each recorded event before sending to the subscriber. concurrency_limit defines the maximum number of concurrent subscribers allowed to connect to the subscription. By default only one subscriber may connect. If too many subscribers attempt to connect to the subscription an {:error, :too_many_subscribers} is returned. transient is an optional boolean flag to create a transient subscription. See subscribe_to_stream for the full information. The subscription will resume from the last acknowledged event if it already exists. It will ignore the start_from argument in this case. Returns {:ok, subscription} when subscription succeeds. Example { :ok , subscription } = EventStore . subscribe_to_all_streams ( &quot;all_subscription&quot; , self ( ) ) # wait for the subscription confirmation receive do { :subscribed , ^ subscription } -&gt; IO . puts &quot;Successfully subscribed to all streams&quot; end receive do { :events , events } -&gt; IO . puts &quot;Received events: &quot; &lt;&gt; inspect ( events ) # acknowledge receipt EventStore . ack ( subscription , events ) end","ref":"EventStore.html#c:subscribe_to_all_streams/3","title":"EventStore.subscribe_to_all_streams/3","type":"callback"},{"doc":"Create a subscription to a single stream. By default the subscription is persistent. The subscriber process will be notified of each batch of events appended to the single stream identified by stream_uuid . stream_uuid is the stream to subscribe to. Use the $all identifier to subscribe to events from all streams. subscription_name is used to uniquely identify the subscription. subscriber is a process that will be sent {:events, events} notification messages. opts is an optional map providing additional subscription configuration: name the name of the event store if provided to start_link/1 . start_from is a pointer to the first event to receive. It must be one of: :origin for all events from the start of the stream (default). :current for any new events appended to the stream after the subscription has been created. any positive integer for a stream version to receive events after. selector to define a function to filter each event, i.e. returns only those elements for which fun returns a truthy value. mapper to define a function to map each recorded event before sending to the subscriber. concurrency_limit defines the maximum number of concurrent subscribers allowed to connect to the subscription. By default only one subscriber may connect. If too many subscribers attempt to connect to the subscription an {:error, :too_many_subscribers} is returned. buffer_size limits how many in-flight events will be sent to the subscriber process before acknowledgement of successful processing. This limits the number of messages sent to the subscriber and stops their message queue from getting filled with events. Defaults to one in-flight event. checkpoint_threshold determines how frequently a checkpoint is written to the database for the subscription after events are acknowledged. Increasing the threshold will reduce the number of database writes for busy subscriptions, but means that events might be replayed when the subscription resumes if the checkpoint cannot be written. The default is to persist the checkpoint after each acknowledgement. checkpoint_after (milliseconds) used to ensure a checkpoint is written after a period of inactivity even if the checkpoint threshold has not been met. This ensures checkpoints are consistently written during less busy periods. It is only applicable when a checkpoint threshold has been set as the default subscription behaviour is to checkpoint after each acknowledgement. partition_by is an optional function used to partition events to subscribers. It can be used to guarantee processing order when multiple subscribers have subscribed to a single subscription. The function is passed a single argument (an EventStore.RecordedEvent struct) and must return the partition key. As an example to guarantee events for a single stream are processed serially, but different streams are processed concurrently, you could use the stream_uuid as the partition key. by_stream = fn % EventStore.RecordedEvent { stream_uuid : stream_uuid } -&gt; stream_uuid end { :ok , _subscription } = EventStore . subscribe_to_stream ( stream_uuid , &quot;example&quot; , self ( ) , concurrency_limit : 10 , partition_by : by_stream ) transient is an optional boolean flag to create a transient subscription. By default this is set to false . If you want to create a transient subscription set this flag to true. Your subscription will not be persisted, so if the subscription is restarted, you will receive the events again starting from start_from . An example usage are short lived event handlers that keep their state in memory but still want to have the guarantee to have received all events. It's possible to create a persistent subscription with some name, stop it and later create a transient subscription with the same name. The transient subscription will now receive all events starting from start_from . If you later stop this transient subscription and start a persistent subscription again with the same name, you will receive the events again as if the transient subscription never existed. The subscription will resume from the last acknowledged event if it already exists. It will ignore the start_from argument in this case. Returns {:ok, subscription} when subscription succeeds. Notification messages Subscribers will initially receive a {:subscribed, subscription} message once the subscription has successfully subscribed. After this message events will be sent to the subscriber, in batches, as {:events, events} where events is a collection of EventStore.RecordedEvent structs. Example { :ok , subscription } = EventStore . subscribe_to_stream ( stream_uuid , &quot;example&quot; , self ( ) ) # wait for the subscription confirmation receive do { :subscribed , ^ subscription } -&gt; IO . puts &quot;Successfully subscribed to stream: &quot; &lt;&gt; inspect ( stream_uuid ) end receive do { :events , events } -&gt; IO . puts &quot;Received events: &quot; &lt;&gt; inspect ( events ) # acknowledge receipt EventStore . ack ( subscription , events ) end Subscription tuning Use the checkpoint_threshold and checkpoint_after options to configure how frequently checkpoints are written to the database. By default a subscription will persist a checkpoint after each acknowledgement. This can cause high write load on the database for busy subscriptions which receive a large number of events. This problem is known as write amplification where each event written to a stream causes many additional writes as subscriptions acknowledge processing of the event. The checkpoint_threshold controls how frequently checkpoints are persisted. Increasing the threshold reduces the number of database writes. For example using a threshold of 100 means that a checkpoint is written at most once for every 100 events processed. The checkpoint_after ensures that a checkpoint will still be written after a period of inactivity even when the threshold has not been met. This ensures bursts of event processing can be safely handled.","ref":"EventStore.html#c:subscribe_to_stream/4","title":"EventStore.subscribe_to_stream/4","type":"callback"},{"doc":"Unsubscribe an existing subscriber from all event notifications. subscription_name is used to identify the existing subscription process to stop. Returns :ok on success.","ref":"EventStore.html#c:unsubscribe_from_all_streams/2","title":"EventStore.unsubscribe_from_all_streams/2","type":"callback"},{"doc":"Unsubscribe an existing subscriber from event notifications. stream_uuid is the stream to unsubscribe from. subscription_name is used to identify the existing subscription process to stop. Returns :ok on success.","ref":"EventStore.html#c:unsubscribe_from_stream/3","title":"EventStore.unsubscribe_from_stream/3","type":"callback"},{"doc":"","ref":"EventStore.html#t:expected_version/0","title":"EventStore.expected_version/0","type":"type"},{"doc":"","ref":"EventStore.html#t:option/0","title":"EventStore.option/0","type":"type"},{"doc":"","ref":"EventStore.html#t:options/0","title":"EventStore.options/0","type":"type"},{"doc":"","ref":"EventStore.html#t:pagination_option/0","title":"EventStore.pagination_option/0","type":"type"},{"doc":"","ref":"EventStore.html#t:pagination_options/0","title":"EventStore.pagination_options/0","type":"type"},{"doc":"","ref":"EventStore.html#t:persistent_subscription_option/0","title":"EventStore.persistent_subscription_option/0","type":"type"},{"doc":"","ref":"EventStore.html#t:persistent_subscription_options/0","title":"EventStore.persistent_subscription_options/0","type":"type"},{"doc":"","ref":"EventStore.html#t:start_from/0","title":"EventStore.start_from/0","type":"type"},{"doc":"","ref":"EventStore.html#t:t/0","title":"EventStore.t/0","type":"type"},{"doc":"","ref":"EventStore.html#t:transient_subscribe_option/0","title":"EventStore.transient_subscribe_option/0","type":"type"},{"doc":"","ref":"EventStore.html#t:transient_subscribe_options/0","title":"EventStore.transient_subscribe_options/0","type":"type"},{"doc":"Provides access to the EventStore configuration.","ref":"EventStore.Config.html","title":"EventStore.Config","type":"module"},{"doc":"Stop the Postgrex process when the database connection is lost. Stopping the process allows a subscription to be notified when it has lost its advisory lock.","ref":"EventStore.Config.html#advisory_locks_postgrex_opts/1","title":"EventStore.Config.advisory_locks_postgrex_opts/1","type":"function"},{"doc":"Get the data type used to store event data and metadata. Supported data types are: &quot;bytea&quot; - Allows storage of binary strings. &quot;jsonb&quot; - Native JSON type, data is stored in a decomposed binary format that makes it slightly slower to input due to added conversion overhead, but significantly faster to process, since no reparsing is needed.","ref":"EventStore.Config.html#column_data_type/2","title":"EventStore.Config.column_data_type/2","type":"function"},{"doc":"","ref":"EventStore.Config.html#default_postgrex_opts/1","title":"EventStore.Config.default_postgrex_opts/1","type":"function"},{"doc":"Get the event store configuration for the environment.","ref":"EventStore.Config.html#get/2","title":"EventStore.Config.get/2","type":"function"},{"doc":"Get the connection pool module for postgrex.","ref":"EventStore.Config.html#get_pool/0","title":"EventStore.Config.get_pool/0","type":"function"},{"doc":"Normalizes the event store configuration.","ref":"EventStore.Config.html#parse/1","title":"EventStore.Config.parse/1","type":"function"},{"doc":"Get the event store configuration for the environment.","ref":"EventStore.Config.html#parsed/2","title":"EventStore.Config.parsed/2","type":"function"},{"doc":"","ref":"EventStore.Config.html#postgrex_notifications_opts/2","title":"EventStore.Config.postgrex_notifications_opts/2","type":"function"},{"doc":"","ref":"EventStore.Config.html#postgrex_opts/2","title":"EventStore.Config.postgrex_opts/2","type":"function"},{"doc":"","ref":"EventStore.Notifications.Publisher.State.html","title":"EventStore.Notifications.Publisher.State","type":"module"},{"doc":"","ref":"EventStore.Notifications.Publisher.State.html#new/1","title":"EventStore.Notifications.Publisher.State.new/1","type":"function"},{"doc":"A page of results from a paginated query.","ref":"EventStore.Page.html","title":"EventStore.Page","type":"module"},{"doc":"","ref":"EventStore.Page.html#total_pages/2","title":"EventStore.Page.total_pages/2","type":"function"},{"doc":"","ref":"EventStore.Page.html#t:t/0","title":"EventStore.Page.t/0","type":"type"},{"doc":"","ref":"EventStore.Page.html#t:t/1","title":"EventStore.Page.t/1","type":"type"},{"doc":"Pub/sub using Elixir's local Registry module.","ref":"EventStore.PubSub.html","title":"EventStore.PubSub","type":"module"},{"doc":"Broadcasts message on given topic.","ref":"EventStore.PubSub.html#broadcast/3","title":"EventStore.PubSub.broadcast/3","type":"function"},{"doc":"Return the child spec.","ref":"EventStore.PubSub.html#child_spec/1","title":"EventStore.PubSub.child_spec/1","type":"function"},{"doc":"Subscribes the caller to the given topic.","ref":"EventStore.PubSub.html#subscribe/3","title":"EventStore.PubSub.subscribe/3","type":"function"},{"doc":"","ref":"EventStore.Streams.StreamInfo.html","title":"EventStore.Streams.StreamInfo","type":"module"},{"doc":"","ref":"EventStore.Streams.StreamInfo.html#new/1","title":"EventStore.Streams.StreamInfo.new/1","type":"function"},{"doc":"","ref":"EventStore.Streams.StreamInfo.html#validate_expected_version/2","title":"EventStore.Streams.StreamInfo.validate_expected_version/2","type":"function"},{"doc":"","ref":"EventStore.Streams.StreamInfo.html#t:t/0","title":"EventStore.Streams.StreamInfo.t/0","type":"type"},{"doc":"","ref":"EventStore.Tasks.Migration.html","title":"EventStore.Tasks.Migration","type":"module"},{"doc":"Task to show the migration status of EventStore","ref":"EventStore.Tasks.Migrations.html","title":"EventStore.Tasks.Migrations","type":"module"},{"doc":"Run task Parameters config: the parsed EventStore config Opts is_mix: set to true if running as part of a Mix task","ref":"EventStore.Tasks.Migrations.html#exec/2","title":"EventStore.Tasks.Migrations.exec/2","type":"function"},{"doc":"Conveniences for writing EventStore related Mix tasks.","ref":"Mix.EventStore.html","title":"Mix.EventStore","type":"module"},{"doc":"Ensures the given module is an EventStore.","ref":"Mix.EventStore.html#ensure_event_store!/2","title":"Mix.EventStore.ensure_event_store!/2","type":"function"},{"doc":"Returns true if module implements behaviour.","ref":"Mix.EventStore.html#implements?/2","title":"Mix.EventStore.implements?/2","type":"function"},{"doc":"Parses the event store option from the given command line args list. If no event store option is given, it is retrieved from the application environment.","ref":"Mix.EventStore.html#parse_event_store/1","title":"Mix.EventStore.parse_event_store/1","type":"function"},{"doc":"Create the database for the EventStore. The event stores to create are the ones specified under the :event_stores option in the current app configuration. However, if the -e option is given, it replaces the :event_stores config. Examples mix event_store . create - e MyApp.EventStore Command line options -e , --eventstore - the event store to create --quiet - do not log output","ref":"Mix.Tasks.EventStore.Create.html","title":"Mix.Tasks.EventStore.Create","type":"task"},{"doc":"Drop the database for the EventStore. The event stores to drop are the ones specified under the :event_stores option in the current app configuration. However, if the -e option is given, it replaces the :event_stores config. Examples mix event_store . drop - e MyApp.EventStore Command line options -e , --eventstore - the event store to drop --quiet - do not log output","ref":"Mix.Tasks.EventStore.Drop.html","title":"Mix.Tasks.EventStore.Drop","type":"task"},{"doc":"Initialize the database for the EventStore. The event stores to initialize are the ones specified under the :event_stores option in the current app configuration. However, if the -e option is given, it replaces the :event_stores config. Examples mix event_store . init - e MyApp.EventStore Command line options -e , --eventstore - the event store to create --quiet - do not log output","ref":"Mix.Tasks.EventStore.Init.html","title":"Mix.Tasks.EventStore.Init","type":"task"},{"doc":"Migrate an existing EventStore database. The event stores to migrate are the ones specified under the :event_stores option in the current app configuration. However, if the -e option is given, it replaces the :event_stores config. Examples mix event_store . migrate - e MyApp.EventStore Command line options -e , --eventstore - the event store to create --quiet - do not log output","ref":"Mix.Tasks.EventStore.Migrate.html","title":"Mix.Tasks.EventStore.Migrate","type":"task"},{"doc":"Show the migration status of an existing EventStore database. The event stores to inspect are the ones specified under the :event_stores option in the current app configuration. However, if the -e option is given, it replaces the :event_stores config. Examples mix event_store . migrations - e MyApp.EventStore Command line options -e , --eventstore - the event store to create --quiet - do not log output","ref":"Mix.Tasks.EventStore.Migrations.html","title":"Mix.Tasks.EventStore.Migrations","type":"task"},{"doc":"EventData contains the data for a single event before being persisted to storage","ref":"EventStore.EventData.html","title":"EventStore.EventData","type":"module"},{"doc":"","ref":"EventStore.EventData.html#fetch/2","title":"EventStore.EventData.fetch/2","type":"function"},{"doc":"","ref":"EventStore.EventData.html#get_and_update/3","title":"EventStore.EventData.get_and_update/3","type":"function"},{"doc":"","ref":"EventStore.EventData.html#t:t/0","title":"EventStore.EventData.t/0","type":"type"},{"doc":"","ref":"EventStore.EventData.html#t:uuid/0","title":"EventStore.EventData.uuid/0","type":"type"},{"doc":"EventStore.RecordedEvent contains the persisted data and metadata for a single event. Events are immutable once recorded. Recorded event fields event_number - position of the event within the stream. This will be identical to the stream_version when fetching events from a single stream. For the $all stream it will be the globally ordered event number. event_id - a globally unique UUID to identify the event. stream_uuid - the original stream identity for the event. stream_version - the original version of the stream for the event. correlation_id - an optional UUID identifier used to correlate related messages. causation_id - an optional UUID identifier used to identify which message you are responding to. data - the deserialized event data. metadata - a deserialized map of event metadata. created_at - a DateTime (in UTC) indicating when the event was created.","ref":"EventStore.RecordedEvent.html","title":"EventStore.RecordedEvent","type":"module"},{"doc":"","ref":"EventStore.RecordedEvent.html#deserialize/2","title":"EventStore.RecordedEvent.deserialize/2","type":"function"},{"doc":"","ref":"EventStore.RecordedEvent.html#fetch/2","title":"EventStore.RecordedEvent.fetch/2","type":"function"},{"doc":"","ref":"EventStore.RecordedEvent.html#get_and_update/3","title":"EventStore.RecordedEvent.get_and_update/3","type":"function"},{"doc":"","ref":"EventStore.RecordedEvent.html#t:t/0","title":"EventStore.RecordedEvent.t/0","type":"type"},{"doc":"","ref":"EventStore.RecordedEvent.html#t:uuid/0","title":"EventStore.RecordedEvent.uuid/0","type":"type"},{"doc":"Snapshot data.","ref":"EventStore.Snapshots.SnapshotData.html","title":"EventStore.Snapshots.SnapshotData","type":"module"},{"doc":"","ref":"EventStore.Snapshots.SnapshotData.html#deserialize/2","title":"EventStore.Snapshots.SnapshotData.deserialize/2","type":"function"},{"doc":"","ref":"EventStore.Snapshots.SnapshotData.html#serialize/2","title":"EventStore.Snapshots.SnapshotData.serialize/2","type":"function"},{"doc":"","ref":"EventStore.Snapshots.SnapshotData.html#t:t/0","title":"EventStore.Snapshots.SnapshotData.t/0","type":"type"},{"doc":"A serializer that uses the JSON format.","ref":"EventStore.JsonSerializer.html","title":"EventStore.JsonSerializer","type":"module"},{"doc":"Deserialize given JSON binary data to the expected type.","ref":"EventStore.JsonSerializer.html#deserialize/2","title":"EventStore.JsonSerializer.deserialize/2","type":"function"},{"doc":"Serialize given term to JSON binary data.","ref":"EventStore.JsonSerializer.html#serialize/1","title":"EventStore.JsonSerializer.serialize/1","type":"function"},{"doc":"Serialize to/from PostgreSQL's native jsonb format.","ref":"EventStore.JsonbSerializer.html","title":"EventStore.JsonbSerializer","type":"module"},{"doc":"Callback implementation for EventStore.Serializer.deserialize/2 .","ref":"EventStore.JsonbSerializer.html#deserialize/2","title":"EventStore.JsonbSerializer.deserialize/2","type":"function"},{"doc":"Callback implementation for EventStore.Serializer.serialize/1 .","ref":"EventStore.JsonbSerializer.html#serialize/1","title":"EventStore.JsonbSerializer.serialize/1","type":"function"},{"doc":"","ref":"EventStore.JsonbSerializer.html#to_struct/2","title":"EventStore.JsonbSerializer.to_struct/2","type":"function"},{"doc":"Specification of a serializer to convert between an Elixir term and its representation in the database.","ref":"EventStore.Serializer.html","title":"EventStore.Serializer","type":"behaviour"},{"doc":"Deserialize the given data to the corresponding term.","ref":"EventStore.Serializer.html#c:deserialize/2","title":"EventStore.Serializer.deserialize/2","type":"callback"},{"doc":"Serialize the given term to a representation that can be stored by the database.","ref":"EventStore.Serializer.html#c:serialize/1","title":"EventStore.Serializer.serialize/1","type":"callback"},{"doc":"Get the serializer module from the given config for the event store.","ref":"EventStore.Serializer.html#serializer/2","title":"EventStore.Serializer.serializer/2","type":"function"},{"doc":"","ref":"EventStore.Serializer.html#t:config/0","title":"EventStore.Serializer.config/0","type":"type"},{"doc":"","ref":"EventStore.Serializer.html#t:t/0","title":"EventStore.Serializer.t/0","type":"type"},{"doc":"A serializer that uses Erlang's external term format ( http://erlang.org/doc/apps/erts/erl_ext_dist.html )","ref":"EventStore.TermSerializer.html","title":"EventStore.TermSerializer","type":"module"},{"doc":"Deserialize given binary data in Erlang's external term format.","ref":"EventStore.TermSerializer.html#deserialize/2","title":"EventStore.TermSerializer.deserialize/2","type":"function"},{"doc":"Serialize given term to binary data.","ref":"EventStore.TermSerializer.html#serialize/1","title":"EventStore.TermSerializer.serialize/1","type":"function"},{"doc":"Task to create the EventStore","ref":"EventStore.Tasks.Create.html","title":"EventStore.Tasks.Create","type":"module"},{"doc":"Runs database and schema create task. Parameters config: the parsed EventStore config Opts is_mix: set to true if running as part of a Mix task quiet: set to true to silence output","ref":"EventStore.Tasks.Create.html#exec/2","title":"EventStore.Tasks.Create.exec/2","type":"function"},{"doc":"Task to drop the EventStore database.","ref":"EventStore.Tasks.Drop.html","title":"EventStore.Tasks.Drop","type":"module"},{"doc":"Runs task Parameters config: the parsed EventStore config Opts is_mix: set to true if running as part of a Mix task quiet: set to true to silence output","ref":"EventStore.Tasks.Drop.html#exec/2","title":"EventStore.Tasks.Drop.exec/2","type":"function"},{"doc":"Task to initalize the EventStore database","ref":"EventStore.Tasks.Init.html","title":"EventStore.Tasks.Init","type":"module"},{"doc":"Runs task Parameters config: the parsed EventStore config Opts is_mix: set to true if running as part of a Mix task quiet: set to true to silence output","ref":"EventStore.Tasks.Init.html#exec/2","title":"EventStore.Tasks.Init.exec/2","type":"function"},{"doc":"Task to migrate EventStore","ref":"EventStore.Tasks.Migrate.html","title":"EventStore.Tasks.Migrate","type":"module"},{"doc":"Run task Parameters config: the parsed EventStore config Opts is_mix: set to true if running as part of a Mix task quiet: set to true to silence output","ref":"EventStore.Tasks.Migrate.html#exec/2","title":"EventStore.Tasks.Migrate.exec/2","type":"function"},{"doc":"","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"List running event store instances ( #244 ). Paginate streams ( #246 ). Add stream_info/2 function ( #247 ). Remove unmaintained elixir_uuid dependency ( #253 ). Add option to use EventStore with PgBouncer ( #249 ).","ref":"changelog.html#v1-4-0","title":"Changelog - v1.4.0","type":"extras"},{"doc":"Enhancements Add postgrex socket_options option ( #242 ). Bug fixes Fix bug with subscriptions trigger in older Postgres versions ( #241 ). Upgrading This release includes a database migration to be run. Please read the Upgrading an EventStore guide for details on how to migrate an existing database.","ref":"changelog.html#v1-3-2","title":"Changelog - v1.3.2","type":"extras"},{"doc":"Bug fixes Support running event store migrations when using a schema ( #239 ).","ref":"changelog.html#v1-3-1","title":"Changelog - v1.3.1","type":"extras"},{"doc":"Improve performance of appending events under normal and degraded network conditions ( #230 ). Subscription checkpoint tuning ( #237 ).","ref":"changelog.html#v1-3-0","title":"Changelog - v1.3.0","type":"extras"},{"doc":"Fix bug with catch-up all streams subscription where the checkpoint is not committed for hard deleted streams ( #238 ). Upgrading This release requires a database migration to be run. Please read the Upgrading an EventStore guide for details on how to migrate an existing database.","ref":"changelog.html#bug-fixes","title":"Changelog - Bug fixes","type":"extras"},{"doc":"Add :configure to postgrex connection options ( #233 ). Use runtime configuration in Mix tasks ( #236 ).","ref":"changelog.html#v1-2-3","title":"Changelog - v1.2.3","type":"extras"},{"doc":"Read stream and stream events backward ( #234 ).","ref":"changelog.html#v1-2-2","title":"Changelog - v1.2.2","type":"extras"},{"doc":"Allow optional event_id to be included in EventStore.EventData struct ( #229 ). Adds an option to supply an existing database connection or transaction to EventStore functions ( #231 ).","ref":"changelog.html#v1-2-1","title":"Changelog - v1.2.1","type":"extras"},{"doc":"Enhancements Delete event stream ( #203 ). Introduce mix event_store.migrations task to list migration status ( #207 ). Remove distributed registry ( #210 ). Hibernate subscription process after inactivity ( #214 ). Runtime event store configuration ( #217 ). Shared database connection pools ( #216 ). Shared database connection for notifications ( #225 ). Transient subscriptions ( #215 ) Improve resilience when database connection is unavailable ( #226 ). Upgrading This release requires a database migration to be run. Please read the Upgrading an EventStore guide for details on how to migrate an existing database. Breaking changes Usage of EventStore.Tasks.Init task to initialise an event store database has been changed as follows: Previous usage: :ok = EventStore.Tasks.Init . exec ( MyApp.EventStore , config , opts ) Usage now: :ok = EventStore.Tasks.Init . exec ( config ) :ok = EventStore.Tasks.Init . exec ( config , opts ) Bug fixes Support appending events to a stream with :any_version concurrently ( #209 ).","ref":"changelog.html#v1-2-0","title":"Changelog - v1.2.0","type":"extras"},{"doc":"Enhancements Support Postgres schemas ( #182 ). Dynamic event store ( #184 ). Add timeout option to config ( #189 ). Namespace advisory lock to prevent clash with other applications ( #166 ). Use database lock to prevent migrations from running concurrently ( #204 ). Breaking changes The following EventStore API functions have been changed where previously (in v1.0 and earlier) the last argument was an optional timeout (a non-negative integer or :infinity ). This has been changed to be an optional Keyword list, which may include a timeout (e.g. [timeout: 5_000] ). The stream_forward and stream_all_forward functions now also require the optional read_batch_size argument to be provided as part of the options Keyword list. These changes were required to support dynamic event stores where an event store name can be included in the options to each function. If you did not provide a timeout to any of these functions then you will not need to make any changes to your code. See the example usages below for details. EventStore.append_to_stream EventStore.link_to_stream EventStore.read_stream_forward EventStore.read_all_streams_forward EventStore.stream_forward EventStore.stream_all_forward Previous usage: EventStore . append_to_stream ( stream_uuid , expected_version , events , timeout ) EventStore . link_to_stream ( stream_uuid , expected_version , events_or_event_ids , timeout ) EventStore . read_stream_forward ( stream_uuid , start_version , count , timeout ) EventStore . read_all_streams_forward ( start_version , count , timeout ) EventStore . stream_forward ( stream_uuid , start_version , read_batch_size , timeout ) EventStore . stream_all_forward ( start_version , read_batch_size , timeout ) Usage now: EventStore . append_to_stream ( stream_uuid , expected_version , events , timeout : timeout ) EventStore . link_to_stream ( stream_uuid , expected_version , events_or_event_ids , timeout : timeout ) EventStore . read_stream_forward ( stream_uuid , start_version , count , timeout : timeout ) EventStore . read_all_streams_forward ( start_version , count , timeout : timeout ) EventStore . stream_forward ( stream_uuid , start_version , read_batch_size : read_batch_size , timeout : timeout ) EventStore . stream_all_forward ( start_version , read_batch_size : read_batch_size , timeout : timeout ) Upgrading This release requires a database migration to be run. Please read the Upgrading an EventStore guide for details on how to migrate an existing database.","ref":"changelog.html#v1-1-0","title":"Changelog - v1.1.0","type":"extras"},{"doc":"Bug fixes Use event's stream version when appending events to a stream ( #202 ).","ref":"changelog.html#v1-0-3","title":"Changelog - v1.0.3","type":"extras"},{"doc":"Enhancements Prevent double supervision by starting / stopping supervisor manually ( #194 ). Use DynamicSupervisor for subscriptions.","ref":"changelog.html#v1-0-2","title":"Changelog - v1.0.2","type":"extras"},{"doc":"Bug fixes Fix EventStore.Registration.DistributedForwarder state when running multiple nodes ( #186 ).","ref":"changelog.html#v1-0-1","title":"Changelog - v1.0.1","type":"extras"},{"doc":"Enhancements Support multiple event stores ( #168 ). Add support for queue_target and queue_interval database connection settings ( #172 ). Add support for created_at values to be of type NaiveDateTime ( #175 ). Bug fixes Fix function clause error on DBConnection.ConnectionError ( #167 ). Upgrading Follow the upgrade guide to define and use your own application specific event store].","ref":"changelog.html#v1-0-0","title":"Changelog - v1.0.0","type":"extras"},{"doc":"Enhancements SSL support including Mix tasks ( #161 ). Use timestamp with time zone for timestamp fields ( #150 ). Upgrading Upgrade your existing EventStore database by running: mix event_store.migrate Note : The migrate command is idempotent and can be safely run multiple times. You can drop and recreate an EventStore database by running: mix do event_store.drop, event_store.create, event_store.init","ref":"changelog.html#v0-17-0","title":"Changelog - v0.17.0","type":"extras"},{"doc":"Bug fixes Fix issue with concurrent subscription partitioning ( #162 ). Reliably start EventStore.Notifications.Supervisor on :global name clash ( #165 ).","ref":"changelog.html#v0-16-2","title":"Changelog - v0.16.2","type":"extras"},{"doc":"Bug fixes Stop Postgrex database connection process in mix event_store.init and event_store.migrate tasks after use to prevent IEx shutdown when tasks are run together (as mix do event_store.init, event_store.migrate ). Ensure the event store application doesn't crash when the database connection is lost ( #159 ).","ref":"changelog.html#v0-16-1","title":"Changelog - v0.16.1","type":"extras"},{"doc":"Enhancements Add :socket and :socket_dir config options ( #132 ). Rename uuid dependency to elixir_uuid ( #135 ). Subscription concurrency ( #134 ). Send :subscribed message to all subscribers connected to a subscription ( #136 ). Update to postgrex v0.14 ( #143 ). Breaking changes Replace :poison with :jason for JSON event data &amp; metadata serialization ( #144 ). To support this change you will need to derive the Jason.Encoder protocol for all of your events. This can be done by adding @derive Jason.Encoder before defining the struct in every event module. defmodule Event1 do @derive Jason.Encoder defstruct [ :id , :data ] end Or using Protocol.derive/2 for each event, as shown below. require Protocol for event &lt;- [ Event1 , Event2 , Event3 ] do Protocol . derive ( Jason.Encoder , event ) end","ref":"changelog.html#v0-16-0","title":"Changelog - v0.16.0","type":"extras"},{"doc":"Enhancements Use a timeout of :infinity for the migration task ( mix event_store.migrate ) to allow database migration to run longer than the default 15 seconds. Bug fixes Socket closing causes the event store to never receive notifications ( #130 ). Subscription with selector function should notify pending events after all filtered ( #131 ).","ref":"changelog.html#0-15-1","title":"Changelog - 0.15.1","type":"extras"},{"doc":"Support system environment variables for all config ( #115 ). Allow subscriptions to filter the events they receive ( #114 ). Allow callers to omit event_type when event data is a struct ( #118 ). Remove dependency on psql for event_store.create , event_store.init , event_store.migrate , and event_store.drop mix tasks ( #117 ). Supports query parameters in URL for database connection ( #119 ). Improve typespecs and include Dialyzer in Travis CI build ( #121 ).","ref":"changelog.html#0-15-0","title":"Changelog - 0.15.0","type":"extras"},{"doc":"Add JSONB support ( #86 ). Add :ssl and :ssl_opts config params ( #88 ). Make mix event_store.init task do nothing if events table already exists ( #89 ). Timeout issue when using EventStore.read_stream_forward ( #92 ). Replace :info level logging with :debug ( #90 ). Dealing better with Poison dependency ( #91 ). Publish events directly to subscriptions ( #93 ). Use PostgreSQL advisory locks to enforce only one subscription instance ( #98 ). Remove stream process ( #99 ). Use PostgreSQL's NOTIFY / LISTEN for event pub/sub ( #100 ). Link existing events to another stream ( #103 ). Subscription notification message once successfully subscribed ( #104 ). Transient subscriptions ( #105 ). Transient subscription event mapping function ( #108 ). Turn EventStore mix tasks into generic tasks for use with Distillery during deployment ( #111 ). Upgrading Upgrade your existing EventStore database by running: mix event_store.migrate You can drop and recreate an EventStore database by running: mix do event_store.drop, event_store.create, event_store.init","ref":"changelog.html#0-14-0","title":"Changelog - 0.14.0","type":"extras"},{"doc":"Bug fixes Use Supervisor.child_spec with an explicit id for Registry processes to support Elixir v1.5.0 and v1.5.1 ( v1.5.2 contains a fix for this issue).","ref":"changelog.html#v0-13-2","title":"Changelog - v0.13.2","type":"extras"},{"doc":"Bug fixes EventStore migrate mix task read migration SQL scripts from app dir ( Application.app_dir(:eventstore) ).","ref":"changelog.html#v0-13-1","title":"Changelog - v0.13.1","type":"extras"},{"doc":"Enhancements Use a UUID field for the event_id column, rename existing field to event_number ( #75 ). Use uuid data type for event correlation_id and causation_id ( #57 ). Mix task to migrate an existing EventStore database ( mix event_store.migrate ). Bug fixes Append to stream is limited to 7,281 events in a single request ( #77 ). Upgrading Upgrade your existing EventStore database by running: mix event_store.migrate Or you can drop and recreate the EventStore database by running: mix do event_store.drop, event_store.create, event_store.init","ref":"changelog.html#v0-13-0","title":"Changelog - v0.13.0","type":"extras"},{"doc":"Bug fixes Publisher only notifies first pending event batch ( #81 ).","ref":"changelog.html#v0-12-1","title":"Changelog - v0.12.1","type":"extras"},{"doc":"Enhancements Allow optimistic concurrency check on write to be optional ( #31 ). Bug fixes Fix issue where subscription doesn't immediately receive events published while transitioning between catch-up and subscribed. Any missed events would be noticed and replayed upon next event publish.","ref":"changelog.html#v0-12-0","title":"Changelog - v0.12.0","type":"extras"},{"doc":"Enhancements Support for running on a cluster of nodes using Swarm for process distribution ( #53 ). Add stream_version column to streams table. It is used for stream info querying and optimistic concurrency checks, instead of querying the events table. Upgrading Run the schema migration v0.11.0.sql script against your event store database.","ref":"changelog.html#v0-11-0","title":"Changelog - v0.11.0","type":"extras"},{"doc":"Bug fixes Fix for ack of last seen event in stream subscription ( #66 ).","ref":"changelog.html#v0-10-1","title":"Changelog - v0.10.1","type":"extras"},{"doc":"Enhancements Writer per event stream ( #55 ). You must run the schema migration v0.10.0.sql script against your event store database. Use DBConnection 's built in support for connection pools (using poolboy).","ref":"changelog.html#v0-10-0","title":"Changelog - v0.10.0","type":"extras"},{"doc":"Enhancements Adds causation_id alongside correlation_id for events ( #48 ). To migrate an existing event store database execute v0.9.0.sql script. Allow single stream, and all streams, subscriptions to provide a mapper function that maps every received event before sending to the subscriber. EventStore . subscribe_to_stream ( stream_uuid , &quot;subscription&quot; , subscriber , mapper : fn event -&gt; event . data end ) Subscribers now receive an {:events, events} tuple and should acknowledge receipt by: EventStore.ack(subscription, events)","ref":"changelog.html#v0-9-0","title":"Changelog - v0.9.0","type":"extras"},{"doc":"Enhancements Add Access functions to EventStore.EventData and EventStore.RecordedEvent modules ( #37 ). Allow database connection URL to be provided as a system variable ( #39 ). Bug fixes Writer not parsing database connection URL from config ( #38 ).","ref":"changelog.html#v0-8-1","title":"Changelog - v0.8.1","type":"extras"},{"doc":"Enhancements Stream events from a single stream forward.","ref":"changelog.html#v0-8-0","title":"Changelog - v0.8.0","type":"extras"},{"doc":"Enhancements Subscriptions use Elixir streams to read events when catching up.","ref":"changelog.html#v0-7-4","title":"Changelog - v0.7.4","type":"extras"},{"doc":"Enhancements Upgrade fsm dependency to v0.3.0 to remove Elixir 1.4 compiler warnings.","ref":"changelog.html#v0-7-3","title":"Changelog - v0.7.3","type":"extras"},{"doc":"Enhancements Stream all events forward ( #34 ).","ref":"changelog.html#v0-7-2","title":"Changelog - v0.7.2","type":"extras"},{"doc":"Enhancements Allow snapshots to be deleted ( #26 ).","ref":"changelog.html#v0-7-1","title":"Changelog - v0.7.1","type":"extras"},{"doc":"Enhancements Subscribe to a single stream, or all streams, from a specified start position ( #17 ).","ref":"changelog.html#v0-7-0","title":"Changelog - v0.7.0","type":"extras"},{"doc":"Bug fixes Subscriptions that are at max capacity should wait until all pending events have been acknowledged by the subscriber being catching up with any unseen events.","ref":"changelog.html#v0-6-2","title":"Changelog - v0.6.2","type":"extras"},{"doc":"Enhancements Use IO lists to build insert events SQL statement ( #23 ).","ref":"changelog.html#v0-6-1","title":"Changelog - v0.6.1","type":"extras"},{"doc":"Enhancements Use NaiveDateTime for each recorded event's created_at property.","ref":"changelog.html#v0-6-0","title":"Changelog - v0.6.0","type":"extras"},{"doc":"Enhancements Provide typespecs for the public API ( #16 ) Fix compilation warnings in mix database task ( #14 ) Bug fixes Read stream forward does not use count to limit returned events ( #10 )","ref":"changelog.html#v0-5-2","title":"Changelog - v0.5.2","type":"extras"},{"doc":"Enhancements Ack handled events in subscribers ( #18 ). Buffer events between publisher and subscriber ( #19 ).","ref":"changelog.html#v0-5-0","title":"Changelog - v0.5.0","type":"extras"},{"doc":"EventStore is available in Hex and can be installed as follows: Add eventstore to your list of dependencies in mix.exs : def deps do [ { :eventstore , &quot;~&gt; 1.4&quot; } ] end Run mix deps.get to install the new dependency. Define an event store module for your application: defmodule MyApp.EventStore do use EventStore , otp_app : :my_app # Optional `init/1` function to modify config at runtime. def init ( config ) do { :ok , config } end end Add a config entry containing the PostgreSQL database connection details for your event store module to each environment's mix config file (e.g. config/dev.exs ): config :my_app , MyApp.EventStore , serializer : EventStore.JsonSerializer , username : &quot;postgres&quot; , password : &quot;postgres&quot; , database : &quot;eventstore&quot; , hostname : &quot;localhost&quot; # OR use a URL to connect instead config :my_app , MyApp.EventStore , serializer : EventStore.JsonSerializer , url : &quot;postgres://postgres:postgres@localhost/eventstore&quot; Note: To use an EventStore with Commanded you should configure the event store to use Commanded's JSON serializer which provides additional support for JSON decoding: config :my_app , MyApp.EventStore , serializer : Commanded.Serialization.JsonSerializer Configure optional database connection settings: config :my_app , MyApp.EventStore , pool_size : 10 queue_target : 50 queue_interval : 1_000 , schema : &quot;schema_name&quot; The database connection pool configuration options are: :pool_size - The number of connections (default: 10 ). Handling requests is done through a queue. When DBConnection is started, there are two relevant options to control the queue: :queue_target - in milliseconds (default: 50ms). :queue_interval - in milliseconds (default: 1,000ms). Additional options: :schema - define the Postgres schema to use (default: public schema). :timeout - set the default database query timeout in milliseconds (default: 15,000ms). :shared_connection_pool - allows a database connection pool to be shared amongst multiple event store instances (default: nil ). Subscription options: :subscription_retry_interval - interval between subscription connection retry attempts (default: 60,000ms). :subscription_hibernate_after - subscriptions will automatically hibernate to save memory after a period of inactivity (default: 15,000ms). Add your event store module to the event_stores list for your app in mix config: # config/config.exs config :my_app , event_stores : [ MyApp.EventStore ] This ensures the event store mix tasks can be run without specifying the event store as a command line argument (e.g. mix event_store.init -e MyApp.EventStore ). Create and initialize the event store database and tables using the mix tasks: $ mix do event_store.create, event_store.init The final piece of configuration is to setup your event store as a supervisor within your application's supervision tree (e.g. in lib/my_app/application.ex , inside the start/2 function): defmodule MyApp.Application do use Application def start ( _type , _args ) do children = [ MyApp.EventStore ] opts = [ strategy : :one_for_one , name : MyApp.Supervisor ] Supervisor . start_link ( children , opts ) end end","ref":"getting-started.html","title":"Getting started","type":"extras"},{"doc":"You can use an existing PostgreSQL database with EventStore by running the following mix task to create and initialize the event store tables: $ mix event_store.init","ref":"getting-started.html#using-an-existing-database","title":"Getting started - Using an existing database","type":"extras"},{"doc":"To drop an existing EventStore database and recreate it you can run the following mix tasks: $ mix do event_store.drop, event_store.create, event_store.init Warning This will drop the database and delete all data.","ref":"getting-started.html#reset-an-existing-database","title":"Getting started - Reset an existing database","type":"extras"},{"doc":"If you're using an Elixir release build by the task mix release you won't have mix available therefore you won't be able to run the following command in order to initialize a new database. $ mix do event_store.create, event_store.init To do that you can use task modules defined inside EventStore (in lib/mix/tasks ): EventStore.Tasks.Create EventStore.Tasks.Init So you can take advantage of the running one-off commands supported by Mix release, using a helper module defined like this: defmodule MyApp.ReleaseTasks do def init_event_store do { :ok , _ } = Application . ensure_all_started ( :postgrex ) { :ok , _ } = Application . ensure_all_started ( :ssl ) :ok = Application . load ( :my_app ) config = MyApp.EventStore . config ( ) :ok = EventStore.Tasks.Create . exec ( config , [ ] ) :ok = EventStore.Tasks.Init . exec ( config , [ ] ) end end","ref":"getting-started.html#initialize-a-database-using-an-elixir-release","title":"Getting started - Initialize a database using an Elixir release","type":"extras"},{"doc":"A Postgres database contains one or more named schemas , which in turn contain tables. By default tables are defined in a schema named &quot;public&quot;. An EventStore can be configured to use a different schema name. Specify the schema when using the EventStore macro in your event store module: defmodule MyApp.EventStore do use EventStore , otp_app : :my_app , schema : &quot;example&quot; end Or provide the schema as an option in the init/1 callback function: defmodule MyApp.EventStore do use EventStore , otp_app : :my_app def init ( config ) do { :ok , Keyword . put ( config , :schema , &quot;example&quot; ) } end end Or define it in environment config when configuring the database connection settings: # config/config.exs config :my_app , MyApp.EventStore , schema : &quot;example&quot; This feature allows you to define and start multiple event stores sharing a single Postgres database, but with their data isolated and segregated by schema. Note the mix event_store.&lt;task&gt; tasks to create, initialize, and drop an event store database will also handle creating and/or dropping the schema.","ref":"getting-started.html#using-postgres-schemas","title":"Getting started - Using Postgres schemas","type":"extras"},{"doc":"EventStore has support for persisting event data and metadata as either: Binary data, using the bytea data type , designed for storing binary strings. JSON data, using the jsonb data type , specifically designed for storing JSON encoded data. The default data type is bytea . This can be used with any binary serializer, such as the Erlang Term format, JSON data encoded to binary, and other serialization formats. Using the jsonb data type The advantage this format is that it allows you to execute ad-hoc SQL queries against the event data or metadata using PostgreSQL's native JSON query support. To enable native JSON support you need to configure your event store to use the jsonb data type. You must also use the EventStore.JsonbSerializer serializer, to ensure event data and metadata is correctly serialized to JSON, and include the Postgres types module ( EventStore.PostgresTypes ) for the Postgrex library to support JSON. # config/config.exs config :my_app , MyApp.EventStore , column_data_type : &quot;jsonb&quot; , serializer : EventStore.JsonbSerializer , types : EventStore.PostgresTypes Finally, you need to include the Jason library as a dependency in mix.exs to enable Postgrex JSON support and then run mix deps.get to install. # mix.exs defp deps do [ { :jason , &quot;~&gt; 1.2&quot; } ] end These settings must be configured before creating the EventStore database. It's not possible to migrate between bytea and jsonb data types once you've created the database. This must be decided in advance.","ref":"getting-started.html#event-data-and-metadata-data-type","title":"Getting started - Event data and metadata data type","type":"extras"},{"doc":"EventStore uses LISTEN/NOTIFY and pg_advisory_locks Postgres capabilities. Unfortunately, they are not compatible with PgBouncer running in transaction (most typical) mode. As a workaround, you can provide an additional session_mode_url parameter to the EventStore config: config :my_app , MyApp.EventStore , url : &quot;postgres://postgres:pgbouncer-in-transaction-mode@localhost/eventstore&quot; session_mode_url : &quot;postgres://postgres:pgbouncer-in-session-mode@localhost/eventstore&quot; This will allow the EventStore to use your regular pool settings to connect to the database defined in url for most database operations. It will separately establish connections using the session_mode_url where necessary which you should point to PgBouncer in session mode or connected directly to the Postgres instance.","ref":"getting-started.html#using-with-pgbouncer","title":"Getting started - Using with PgBouncer","type":"extras"},{"doc":"First you need to define your own event store module: defmodule MyApp.EventStore do use EventStore , otp_app : :my_app end","ref":"usage.html","title":"Using the EventStore","type":"extras"},{"doc":"Create a unique identity for each stream. It must be a string. This example uses the elixir_uuid package. stream_uuid = EventStore.UUID . uuid4 ( ) Set the expected version of the stream. This is used for optimistic concurrency. A new stream will be created when the expected version is zero. expected_version = 0 Build a list of events to persist. The data and metadata fields will be serialized to binary data. This uses your own serializer, as defined in config, that implements the EventStore.Serializer behaviour. alias EventStore.EventData alias MyApp.EventStore defmodule ExampleEvent do defstruct [ :key ] end events = [ % EventData { event_type : &quot;Elixir.ExampleEvent&quot; , data : % ExampleEvent { key : &quot;value&quot; } , metadata : %{ user : &quot;someuser@example.com&quot; } , } ] Append the events to the stream: :ok = EventStore . append_to_stream ( stream_uuid , expected_version , events ) Appending events to an existing stream The expected version should equal the number of events already persisted to the stream when appending to an existing stream. This can be set as the length of events returned from reading the stream: alias MyApp.EventStore events = stream_uuid |&gt; EventStore . stream_forward ( ) |&gt; Enum . to_list ( ) stream_version = length ( events ) Append new events to the existing stream: alias EventStore.EventData alias MyApp.EventStore new_events = [ % EventData { .. } , ... ] :ok = EventStore . append_to_stream ( stream_uuid , stream_version , new_events ) Why must you provide the expected stream version? This is to ensure that no events have been appended to the stream by another process between your read and subsequent write. The EventStore.append_to_stream/4 function will return {:error, :wrong_expected_version} when the version you provide is mismatched with the stream. You can resolve this error by reading the stream's events again, then attempt to append your new events using the latest stream version. Optional concurrency check You can choose to append events to a stream without using the concurrency check, or having first read them from the stream, by using one of the following values instead of the expected version: :any_version - No concurrency checking; allow any stream version (including no stream). :no_stream - Ensure the stream does not exist. :stream_exists - Ensure the stream exists. alias MyApp.EventStore :ok = EventStore . append_to_stream ( stream_uuid , :any_version , events )","ref":"usage.html#writing-to-a-stream","title":"Using the EventStore - Writing to a stream","type":"extras"},{"doc":"Read all events from a single stream, starting at the stream's first event: alias MyApp.EventStore { :ok , events } = EventStore . read_stream_forward ( stream_uuid )","ref":"usage.html#reading-from-a-stream","title":"Using the EventStore - Reading from a stream","type":"extras"},{"doc":"Read all events from all streams: alias MyApp.EventStore { :ok , events } = EventStore . read_all_streams_forward ( ) By default this will be limited to read the first 1,000 events from all streams only.","ref":"usage.html#reading-from-all-streams","title":"Using the EventStore - Reading from all streams","type":"extras"},{"doc":"Stream all events from all streams: alias MyApp.EventStore all_events = EventStore . stream_all_forward ( ) |&gt; Enum . to_list ( ) This will read all events into memory, it is for illustration only. Use the Stream functions to process the events in a memory efficient way.","ref":"usage.html#stream-from-all-streams","title":"Using the EventStore - Stream from all streams","type":"extras"},{"doc":"Event linking allows you to include events in multiple streams, such as copying an event from one stream to another, but only a reference to the original event is stored. An event may be present in a stream only once, but may be linked into as many streams as required. Linked events are used to build the $all stream containing every persisted event, globally ordered. Use each recorded event's event_number field for the position of the event within the read/received stream. The stream_uuid and stream_version fields refer to the event's original stream. Read source events: alias MyApp.EventStore { :ok , events } = EventStore . read_stream_forward ( source_stream_uuid ) Link read events to another stream: alias MyApp.EventStore :ok = EventStore . link_to_stream ( target_stream_uuid , 0 , events ) You can also pass a list of event_ids instead of recorded event structs to link events.","ref":"usage.html#linking-events-between-streams","title":"Using the EventStore - Linking events between streams","type":"extras"},{"doc":"There are two types of subscriptions provided by EventStore: Transient subscriptions where new events are broadcast to subscribers immediately after they have been appended to storage. Persistent subscriptions which guarantee at-least-once delivery of every persisted event, provide back-pressure, and can be started, paused, and resumed from any position, including from the stream's origin.","ref":"subscriptions.html","title":"Subscriptions","type":"extras"},{"doc":"PostgreSQL's LISTEN and NOTIFY commands are used to pub/sub event notifications from the database. An after update trigger on the streams table is used to execute NOTIFY for each batch of inserted events. The notification payload contains the stream uuid, stream id, and first / last stream versions (e.g. stream-12345,1,1,5 ). A single process will connect to the database to listen for these notifications. It fetches the event data and broadcasts to all interested subscriptions. This approach supports running an EventStore on multiple nodes, regardless of whether they are connected together to form a cluster using distributed Erlang. One connection per node is used for single node and multi-node deployments.","ref":"subscriptions.html#event-pub-sub","title":"Subscriptions - Event pub/sub","type":"extras"},{"doc":"Use EventStore.subscribe/2 to create a transient subscription to a single stream identified by its stream_uuid . Events will be received in batches as an {:events, events} message, where events is a collection of EventStore.RecordedEvent structs. You can use $all as the stream identity to subscribe to events appended to all streams. With transient subscriptions you do not need to acknowledge receipt of the published events. The subscription will terminate when the subscriber process stops running. Subscribe to single stream events Subscribe to events appended to a single stream: alias MyApp.EventStore :ok = EventStore . subscribe ( stream_uuid ) # receive first batch of events receive do { :events , events } -&gt; IO . puts ( &quot;Received events: &quot; &lt;&gt; inspect ( events ) ) end Filtering events You can provide an event selector function that filters each RecordedEvent before sending it to the subscriber: alias EventStore.RecordedEvent alias MyApp.EventStore EventStore . subscribe ( stream_uuid , selector : fn % RecordedEvent { data : data } -&gt; data != nil end ) # receive first batch of mapped event data receive do { :events , % RecordedEvent { } = event_data } -&gt; IO . puts ( &quot;Received non nil event data: &quot; &lt;&gt; inspect ( event_data ) ) end Mapping events You can provide an event mapping function that maps each RecordedEvent before sending it to the subscriber: alias EventStore.RecordedEvent alias MyApp.EventStore EventStore . subscribe ( stream_uuid , mapper : fn % RecordedEvent { data : data } -&gt; data end ) # receive first batch of mapped event data receive do { :events , event_data } -&gt; IO . puts ( &quot;Received event data: &quot; &lt;&gt; inspect ( event_data ) ) end","ref":"subscriptions.html#transient-subscriptions","title":"Subscriptions - Transient subscriptions","type":"extras"},{"doc":"Persistent subscriptions to a stream will guarantee at least once delivery of every persisted event. Each subscription may be independently paused, then later resumed from where it stopped. The last received and acknowledged event is stored by the EventStore to support resuming at a later time or whenever the subscriber process restarts. A subscription can be created to receive events appended to a single or all streams. Subscriptions must be uniquely named. By default a subscription only supports a single subscriber. Attempting to connect two subscribers to the same subscription will return {:error, :subscription_already_exists} . You can optionally create a competing consumer subscription with multiple subscribers . :subscribed message Once the subscription has successfully subscribed to the stream it will send the subscriber a {:subscribed, subscription} message. This indicates the subscription succeeded and you will begin receiving events. Only one instance of a named subscription to a stream can connect to the database. This guarantees that starting the same subscription on each node when run on a cluster, or when running multiple single instance nodes, will only allow one subscription to actually connect. Therefore you can defer any initialisation until receipt of the {:subscribed, subscription} message to prevent duplicate effort by multiple nodes racing to create or subscribe to the same subscription. :events message For each batch of events appended to the event store your subscriber will receive a {:events, events} message. The events list is a collection of EventStore.RecordedEvent structs. Subscription start from By default subscriptions are created from the stream origin; they will receive all events from the stream. You can optionally specify a given start position: :origin - subscribe to events from the start of the stream (identical to using 0 ). This is the default behaviour. :current - subscribe to events from the current version. event_number (integer) - specify an exact event number to subscribe from. This will be the same as the stream version for single stream subscriptions. Acknowledge received events Receipt of each event by the subscriber must be acknowledged. This allows the subscription to resume on failure without missing an event and to indicate the subscription is ready to receive the next event. The subscriber receives an {:events, events} tuple containing the published events. A subscription is returned when subscribing to the stream. This should be used to send the acknowledgement to using the EventStore.ack/2 function: alias MyApp.EventStore :ok = EventStore . ack ( subscription , events ) A subscriber can confirm receipt of each event in a batch by sending multiple acks, one per event. The subscriber may confirm receipt of the last event in the batch in a single ack. A subscriber will not receive further published events until it has confirmed receipt of all received events. This provides back pressure to the subscription to prevent the subscriber from being overwhelmed with messages if it cannot keep up. The subscription will buffer events until the subscriber is ready to receive, or an overflow occurs. At which point it will move into a catch-up mode and query events and replay them from storage until caught up. Subscribe to all events Subscribe to events appended to all streams: alias MyApp.EventStore { :ok , subscription } = EventStore . subscribe_to_all_streams ( &quot;example_all_subscription&quot; , self ( ) ) # Wait for the subscription confirmation receive do { :subscribed , ^ subscription } -&gt; IO . puts ( &quot;Successfully subscribed to all streams&quot; ) end receive do { :events , events } -&gt; IO . puts &quot;Received events: \#{ inspect events } &quot; # Acknowledge receipt :ok = EventStore . ack ( subscription , events ) end Unsubscribe from all streams: alias MyApp.EventStore :ok = EventStore . unsubscribe_from_all_streams ( &quot;example_all_subscription&quot; ) Subscribe to single stream events Subscribe to events appended to a single stream: alias MyApp.EventStore stream_uuid = EventStore.UUID . uuid4 ( ) { :ok , subscription } = EventStore . subscribe_to_stream ( stream_uuid , &quot;example_single_subscription&quot; , self ( ) ) # Wait for the subscription confirmation receive do { :subscribed , ^ subscription } -&gt; IO . puts ( &quot;Successfully subscribed to single stream&quot; ) end receive do { :events , events } -&gt; # Process events &amp; acknowledge receipt :ok = EventStore . ack ( subscription , events ) end Unsubscribe from a single stream: alias MyApp.EventStore :ok = EventStore . unsubscribe_from_stream ( stream_uuid , &quot;example_single_subscription&quot; ) Start subscription from a given position You can choose to receive events from a given starting position. The supported options are: :origin - Start receiving events from the beginning of the stream or all streams (default). :current - Subscribe to newly appended events only, skipping already persisted events. event_number (integer) - Specify an exact event number to subscribe from. This will be the same as the stream version for single stream subscriptions. Example all stream subscription that will receive new events appended after the subscription has been created: alias MyApp.EventStore { :ok , subscription } = EventStore . subscribe_to_all_streams ( &quot;example_subscription&quot; , self ( ) , start_from : :current ) Event Filtering You can provide an event selector function that run in the subscription process, before sending the event to your mapper and subscriber. You can use this to filter events before dispatching to a subscriber. Subscribe to all streams and provide a selector function that only sends data that the selector function returns true for. alias EventStore.RecordedEvent alias MyApp.EventStore selector = fn % RecordedEvent { event_number : event_number } -&gt; rem ( event_number ) == 0 end { :ok , subscription } = EventStore . subscribe_to_all_streams ( &quot;example_subscription&quot; , self ( ) , selector : selector ) # wait for the subscription confirmation receive do { :subscribed , ^ subscription } -&gt; IO . puts ( &quot;Successfully subscribed to all streams&quot; ) end receive do { :events , filtered_events } -&gt; # ... process events &amp; ack receipt using last `event_number` RecordedEvent { event_number : event_number } = List . last ( filtered_events ) :ok = EventStore . ack ( subscription , event_number ) end Mapping events You can provide an event mapping function that runs in the subscription process, before sending the event to your subscriber. You can use this to change the data received. Subscribe to all streams and provide a mapper function that sends only the event data: alias EventStore.RecordedEvent alias MyApp.EventStore mapper = fn % RecordedEvent { event_number : event_number , data : data } -&gt; { event_number , data } end { :ok , subscription } = EventStore . subscribe_to_all_streams ( &quot;example_subscription&quot; , self ( ) , mapper : mapper ) # wait for the subscription confirmation receive do { :subscribed , ^ subscription } -&gt; IO . puts ( &quot;Successfully subscribed to all streams&quot; ) end receive do { :events , mapped_events } -&gt; # ... process events &amp; ack receipt using last `event_number` { event_number , _data } = List . last ( mapped_events ) :ok = EventStore . ack ( subscription , event_number ) end Subscription concurrency A single persistent subscription can support multiple subscribers. Events will be distributed to subscribers evenly using a round-robin algorithm. The competing consumers pattern enables multiple subscribers to process events concurrently to optimise throughput, to improve scalability and availability, and to balance the workload. By default a subscription will only allow a single subscriber but you can opt-in to concurrent subscriptions be providing a non-negative concurrency_limit as a subscription option. Subscription concurrency configuration options concurrency_limit defines the maximum number of concurrent subscribers allowed to connect to the subscription. By default only one subscriber may connect. If too many subscribers attempt to connect to the subscription an {:error, :too_many_subscribers} is returned. buffer_size limits how many in-flight events will be sent to the subscriber process before acknowledgement of successful processing. This limits the number of messages sent to the subscriber and stops their message queue from getting filled with events. Defaults to one in-flight event. partition_by is an optional function used to partition events to subscribers. It can be used to guarantee processing order when multiple subscribers have subscribed to a single subscription as described in Ordering guarantee below. The function is passed a single argument (an EventStore.RecordedEvent struct) and must return the partition key. As an example to guarantee events for a single stream are processed serially, but different streams are processed concurrently, you could use the stream_uuid as the partition key. Ordering guarantee With multiple subscriber processes connected to a single subscription the ordering of event processing is no longer guaranteed since events may be processed in differing amounts of time. This can cause problems if your event handling code expects events to be processed in the order they were originally appended to a steam. You can use a partition_by function to guarantee ordering of events within a particular group (e.g. per stream) but still allow events for different groups to be processed concurrently. Partitioning gives you the benefits of competing consumers but still allows event ordering by partition where required. Partition by example alias EventStore.RecordedEvent alias MyApp.EventStore by_stream = fn % RecordedEvent { stream_uuid : stream_uuid } -&gt; stream_uuid end { :ok , _subscription } = EventStore . subscribe_to_stream ( stream_uuid , &quot;example&quot; , self ( ) , concurrency_limit : 10 , partition_by : by_stream ) The above subscription would ensure that events for each stream are processed serially (by a single subscriber) in the order they were appended to the stream, but events for any other stream can be processed concurrently by another subscriber. Example persistent subscriber Use a GenServer process to subscribe to the event store and track all notified events: # An example subscriber defmodule Subscriber do use GenServer alias MyApp.EventStore def start_link do GenServer . start_link ( __MODULE__ , [ ] ) end def received_events ( subscriber ) do GenServer . call ( subscriber , :received_events ) end def init ( events ) do # Subscribe to events from all streams { :ok , subscription } = EventStore . subscribe_to_all_streams ( &quot;example_subscription&quot; , self ( ) ) { :ok , %{ events : events , subscription : subscription } } end # Successfully subscribed to all streams def handle_info ( { :subscribed , subscription } , %{ subscription : subscription } = state ) do { :noreply , state } end # Event notification def handle_info ( { :events , events } , state ) do %{ events : existing_events , subscription : subscription } = state # Confirm receipt of received events :ok = EventStore . ack ( subscription , events ) { :noreply , %{ state | events : existing_events ++ events } } end def handle_call ( :received_events , _from , %{ events : events } = state ) do { :reply , events , state } end end Start your subscriber process, which subscribes to all streams in the event store: { :ok , subscriber } = Subscriber . start_link ( ) Deleting a persistent subscription You can delete a single stream or all stream subscription without requiring an active subscriber: alias MyApp.EventStore :ok = EventStore . delete_subscription ( stream_uuid , subscription_name ) :ok = EventStore . delete_all_streams_subscription ( subscription_name ) Deleting the subscription will remove the subscription checkpoint allowing you to later create a subscription with the same name, using any start point. If there is an active subscriber when deleting the subscription it will be stopped.","ref":"subscriptions.html#persistent-subscriptions","title":"Subscriptions - Persistent subscriptions","type":"extras"},{"doc":"The default serialization of event data and metadata uses Erlang's external term format . This is not a recommended serialization format for production usage as backwards compatibility is only guaranteed going back at least two major releases. You must implement the EventStore.Serializer behaviour to provide your preferred serialization format.","ref":"event-serialization.html","title":"Event serialization","type":"extras"},{"doc":"EventStore includes a JSON serializer using Jason in the EventStore.JsonSerializer module. To include it, add {:jason, &quot;~&gt; 1.1&quot;} to your application's mix dependencies and configure your EventStore as below. config :eventstore , MyApp.EventStore , serializer : EventStore.JsonSerializer","ref":"event-serialization.html#json-serialization-using-jason","title":"Event serialization - JSON serialization using Jason","type":"extras"},{"doc":"The example serializer below serializes event data and metadata to JSON using the Poison library. defmodule JsonSerializer do @moduledoc &quot;&quot;&quot; A serializer that uses the JSON format. &quot;&quot;&quot; @behaviour EventStore.Serializer @doc &quot;&quot;&quot; Serialize given term to JSON binary data. &quot;&quot;&quot; def serialize ( term ) do Poison . encode! ( term ) end @doc &quot;&quot;&quot; Deserialize given JSON binary data to the expected type. &quot;&quot;&quot; def deserialize ( binary , config ) do type = case Keyword . get ( config , :type , nil ) do nil -&gt; [ ] type -&gt; type |&gt; String . to_existing_atom |&gt; struct end Poison . decode! ( binary , as : type ) end end Configure the JSON serializer for your event store by setting the serializer option in the mix environment configuration file (e.g. config/dev.exs ): config :eventstore , MyApp.EventStore , serializer : JsonSerializer You must set the event_type field to a string representing the type of event being persisted when using this serializer: % EventStore.EventData { event_type : &quot;Elixir.ExampleEvent&quot; , data : % ExampleEvent { key : &quot;value&quot; } , metadata : %{ user : &quot;someuser@example.com&quot; } , } You can use Atom.to_string/1 to get a string representation of a given event struct compatible with the example JsonSerializer module: event = % ExampleEvent { key : &quot;value&quot; } event_type = Atom . to_string ( event . __struct__ ) #=&gt; &quot;Elixir.ExampleEvent&quot;","ref":"event-serialization.html#example-json-serializer-using-poison","title":"Event serialization - Example JSON serializer using Poison","type":"extras"},{"doc":"EventStore supports running on multiple nodes as either a distributed Erlang cluster or as multiple single instance nodes.","ref":"cluster.html","title":"Running on a cluster of nodes","type":"extras"},{"doc":"PostgreSQL's LISTEN / NOTIFY is used to pub/sub event notifications. A listener database connection process is started on each node. It connects to the database to listen for events and publishes them to interested subscription processes running on the node. The approach is the same regardless of whether distributed Erlang is used or not.","ref":"cluster.html#event-publication","title":"Running on a cluster of nodes - Event publication","type":"extras"},{"doc":"PostgreSQL's advisory locks are used to limit each uniquely named subscription to run at most once. This prevents multiple instances of a subscription from running on different nodes. Advisory locks are faster than table locks, are stored in memory to avoid table bloat, and are automatically cleaned up by the server at the end of the session.","ref":"cluster.html#subscriptions","title":"Running on a cluster of nodes - Subscriptions","type":"extras"},{"doc":"You can use libcluster to automatically form clusters of Erlang nodes, with either static or dynamic node membership. You will need to include libcluster as an additional dependency in mix.exs : defp deps do [ { :libcluster , &quot;~&gt; 2.2&quot; } ] end Then configure your preferred cluster topology in the environment config (e.g. config/config.exs ). An example is shown below using the standard Erlang epmd daemon strategy: config :libcluster , topologies : [ example : [ strategy : Cluster.Strategy.Epmd , config : [ hosts : [ :&quot;node1@127.0.0.1&quot; , :&quot;node2@127.0.0.1&quot; , :&quot;node3@127.0.0.1&quot; ] ] , ] ] Please refer to the libcluster documentation for more detail. Starting a cluster Run an Erlang Port Mapper Daemon (epmd): $ epmd -d Start an iex console per node: $ MIX_ENV=distributed iex --name node1@127.0.0.1 -S mix $ MIX_ENV=distributed iex --name node2@127.0.0.1 -S mix $ MIX_ENV=distributed iex --name node3@127.0.0.1 -S mix The cluster will be automatically formed as soon as the nodes start.","ref":"cluster.html#automatic-cluster-formation","title":"Running on a cluster of nodes - Automatic cluster formation","type":"extras"},{"doc":"Instead of using libcluster you can configure the :kernel application to wait for cluster formation before starting your application during node start up. This approach is useful when you have a static cluster topology that can be defined in config. The sync_nodes_optional configuration specifies which nodes to attempt to connect to within the sync_nodes_timeout window, defined in milliseconds, before continuing with startup. There is also a sync_nodes_mandatory setting which can be used to enforce all nodes are connected within the timeout window or else the node terminates. Each node requires its own individual configuration, listing the other nodes in the cluster: # node1 config config :kernel , sync_nodes_optional : [ :&quot;node2@192.168.1.1&quot; , :&quot;node3@192.168.1.2&quot; ] , sync_nodes_timeout : 30_000 The sync_nodes_timeout can be configured as :infinity to wait indefinitely for all nodes to connect. All involved nodes must have the same value for sync_nodes_timeout . The above approach will only work for Elixir releases. You will need to use Erlang's sys.config file for development purposes. The Erlang equivalent of the :kernerl mix config, as above, is: % node1.sys.config [ { kernel , [ { sync_nodes_optional , [ &#39;node2@127.0.0.1&#39; , &#39;node3@127.0.0.1&#39; ] } , { sync_nodes_timeout , 30000 } ] } ] . Starting a cluster Run an Erlang Port Mapper Daemon (epmd): $ epmd -d Start an iex console per node: $ MIX_ENV=distributed iex --name node1@127.0.0.1 --erl &quot;-config cluster/node1.sys.config&quot; -S mix $ MIX_ENV=distributed iex --name node2@127.0.0.1 --erl &quot;-config cluster/node2.sys.config&quot; -S mix $ MIX_ENV=distributed iex --name node3@127.0.0.1 --erl &quot;-config cluster/node3.sys.config&quot; -S mix The node specific &lt;node&gt;.sys.config files ensure the cluster is formed before starting your application, assuming this occurs within the 30 seconds timeout. Once the cluster has formed, you can use your event store module from any node.","ref":"cluster.html#static-cluster-topology-and-formation","title":"Running on a cluster of nodes - Static cluster topology and formation","type":"extras"},{"doc":"Using the event store when run on a cluster of nodes is identical to single node usage. You can subscribe to a stream, or all streams, on one node and append events to the stream on another. The subscription will be notified of the appended events. Append events to a stream alias EventStore.EventData alias MyApp.EventStore defmodule ExampleEvent do defstruct [ :key ] end stream_uuid = EventStore.UUID . uuid4 ( ) events = [ % EventData { event_type : &quot;Elixir.ExampleEvent&quot; , data : % ExampleEvent { key : &quot;value&quot; } , metadata : %{ user : &quot;someuser@example.com&quot; } , } ] :ok = EventStore . append_to_stream ( stream_uuid , 0 , events ) Read all events alias MyApp.EventStore recorded_events = EventStore . stream_all_forward ( ) |&gt; Enum . to_list ( ) Subscribe to all Streams alias MyApp.EventStore { :ok , subscription } = EventStore . subscribe_to_all_streams ( &quot;example-subscription&quot; , self ( ) , start_from : :origin ) receive do { :subscribed , ^ subscription } -&gt; IO . puts ( &quot;Successfully subscribed to all streams&quot; ) end receive do { :events , events } -&gt; IO . puts ( &quot;Received events: \#{ inspect ( events ) } &quot; ) :ok = EventStore . ack ( subscription , events ) end","ref":"cluster.html#usage","title":"Running on a cluster of nodes - Usage","type":"extras"},{"doc":"The CHANGELOG is used to indicate when a schema migration is required for a given version of the EventStore.","ref":"upgrades.html","title":"Upgrading an EventStore","type":"extras"},{"doc":"You can upgrade an existing EventStore database using the following mix task: mix event_store.migrate The above command expects you to have configured the event store(s) for your application, as shown in the following example: # config/config.exs config :my_app , event_stores : [ MyApp.EventStore ] Or use mix event_store.migrate -e MyApp.EventStore to specify an event store as an argument. Run this command each time you need to upgrade. The command is idempotent and will only run pending migrations if there are any and can be safely run multiple times. It can also be run concurrently as a database lock is used to ensure only one migration is run at at time. It is always worth taking a full backup of the EventStore database before applying an upgrade. Creating an EventStore database, using the mix event_store.create task, will always use the latest database schema.","ref":"upgrades.html#migrating-a-database-with-mix","title":"Upgrading an EventStore - Migrating a database with mix","type":"extras"},{"doc":"If you're using an Elixir release built with mix release you won't have mix available and won't be able to run the above command to migrate the database. Instead you can use the EventStore.Tasks.Migrate task module and running one-off commands supported by Mix release, using a helper module defined like this: defmodule MyApp.ReleaseTasks do def migrate_event_store do { :ok , _ } = Application . ensure_all_started ( :postgrex ) { :ok , _ } = Application . ensure_all_started ( :ssl ) :ok = Application . load ( :my_app ) config = MyApp.EventStore . config ( ) :ok = EventStore.Tasks.Migrate . exec ( config , [ ] ) end end Then run: bin/RELEASE_NAME eval &quot;MyApp.ReleaseTasks.migrate_event_store()&quot;","ref":"upgrades.html#migrating-a-database-using-an-elixir-release","title":"Upgrading an EventStore - Migrating a database using an Elixir release","type":"extras"},{"doc":"In v0.17.x and earlier EventStore was a singleton application with all functionality exposed by the EventStore module. To support multiple event stores in a single Elixir application you must now define your own event store module using the EventStore macro. You can define as many event stores as you like, each one is isolated and can be configured to use its own Postgres database. Getting started First, you must define your own event store module using the EventStore macro: # lib/my_app/event_store.ex defmodule MyApp.EventStore do use EventStore , otp_app : :my_app # Optional `init/1` function to modify config at runtime. def init ( config ) do { :ok , config } end end You can name your event store module however you like and optionally implement an init/1 callback function. Secondly, configure the MyApp.EventStore module (in config/config.exs or each environment's config): config :my_app , MyApp.EventStore , serializer : EventStore.JsonSerializer , username : &quot;postgres&quot; , password : &quot;postgres&quot; , database : &quot;myapp_eventstore&quot; , hostname : &quot;localhost&quot; , pool_size : 10 Note: To use an EventStore with Commanded you should configure the event store to use Commanded's JSON serializer which provides additional support for JSON decoding: config :my_app , MyApp.EventStore , serializer : Commanded.Serialization.JsonSerializer Finally, the event store module must be included within your application's supervision tree, inside the start/2 function: # lib/my_app/application.ex defmodule MyApp.Application do use Application def start ( _type , _args ) do children = [ MyApp.EventStore ] opts = [ strategy : :one_for_one , name : MyApp.Supervisor ] Supervisor . start_link ( children , opts ) end end Optionally, you can configure the event store modules in config/config.exs to allow running the event store mix tasks without providing the event store module as a command line argument: config :my_app , event_stores : [ MyApp.EventStore ] The above configuration allows you to run mix event_store.init instead of mix event_store.init -e MyApp.EventStore (as an example). Usage Use your event store module exactly as you would have previously used the EventStore itself. :ok = MyApp.EventStore . append_to_stream ( stream_uuid , expected_version , events ) For ease of upgrading you can alias your own event store module as EventStore allowing you to use it without making any further code changes: alias MyApp.EventStore , as : EventStore :ok = EventStore . append_to_stream ( stream_uuid , expected_version , events )","ref":"0.17-1.0.html","title":"Upgrade guide v0.17.x to 1.0","type":"extras"}]